<?php 
if( !class_exists('CI_Category') ):
class CI_Category extends WP_Widget {

	function __construct() {
		$widget_ops  = array( 'description' => __( 'Displays a number of the latest (or random) posts from a specific category.', 'ci_theme' ) );
		$control_ops = array( /*'width' => 300, 'height' => 400*/ );
		parent::__construct( 'ci-category', __( '-= CI Category =-', 'ci_theme' ), $widget_ops, $control_ops );
	}


	function widget($args, $instance) {
		extract($args);
		$title = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );

		$category_id = $instance['category_id'];
		$sort        = $instance['sort'];
		$count       = $instance['count'];
		$columns     = $instance['columns'];

		if ( 0 == $count ) {
			return;
		}

		$item_classes = '';
		switch( $columns ) {
			case 2:
				$item_classes = 'col-sm-6';
				break;
			case 3:
				$item_classes = 'col-md-4';
				break;
			case 4:
				$item_classes = 'col-sm-6 col-md-3';
				break;
			default:
				$item_classes = 'col-md-4';
				break;
		}


		echo $before_widget;

		$category = get_category( $category_id );
		$icon     = ci_theme_get_term_icon( $category_id );

		if ( $title ) {
			/* translators: %1$s is replaced by an icon, while %2$s is replaced by text. %1$s may be empty. (E.g. <i class="fa fa-book"> Blog) */
			$output = sprintf( _x( '%1$s %2$s', 'section title', 'ci_theme' ), $icon, $title );
			echo $before_title . $output . $after_title;
		} else {
			if ( ! empty( $category ) && ! is_wp_error( $category ) ) {
				/* translators: %1$s is replaced by an icon, while %2$s is replaced by text. %1$s may be empty. (E.g. <i class="fa fa-book"> Blog) */
				$output = sprintf( _x( '%1$s %2$s', 'section title', 'ci_theme' ), $icon, $category->name );
				echo $before_title . $output . $after_title;
			}
		}


		if ( in_array( $id, array( 'frontpage-widgets' ) ) ) {
			if ( ! empty( $category ) && ! is_wp_error( $category ) ) {
				// The 'parent' and 'child_of' arguments don't work correctly with 'pad_count' and 'orderby' => 'count' at the same time.
				// So we need to sort manually.
				$subcategories = get_terms( 'category', array(
					'child_of'   => $category_id,
					'pad_counts' => true,
				) );

				usort( $subcategories, array( $this, '_sort_subcategories_by_count' ) );
				$subcategories = array_reverse( $subcategories );

				if ( count( $subcategories ) > 0 ) {
					?>
					<span class="entry-categories">
						<?php
							$i = 0;
							foreach ( $subcategories as $subcategory ) {
								$i ++;
								?>
								<a href="<?php echo get_term_link( $subcategory ); ?>"><?php echo $subcategory->name; ?></a> &bull; <?php
								if ( 2 == $i ) {
									break;
								}
							}
						?>
						<a href="<?php echo get_term_link( $category ); ?>"><?php _e( 'All Categories', 'ci_theme' ); ?></a>
					</span>
				<?php
				}

			}
		}


		switch( $sort ) {
			case 'rand':
				$order = array(
					'orderby' => 'rand'
				);
				break;
			case 'rating_num_desc':
				$order = array(
					'meta_key' => 'rating',
					'orderby'  => 'meta_value_num',
					'order'    => 'DESC',
				);
				break;
			case 'rating_num_asc':
				$order = array(
					'meta_key' => 'rating',
					'orderby'  => 'meta_value_num',
					'order'    => 'ASC',
				);
				break;
			case 'rating_asc':
				$order = array(
					'meta_key' => 'rating',
					'orderby'  => 'meta_value',
					'order'    => 'ASC',
				);
				break;
			case 'rating_desc':
				$order = array(
					'meta_key' => 'rating',
					'orderby'  => 'meta_value',
					'order'    => 'DESC',
				);
				break;
			case 'date_asc':
				$order = array(
					'orderby' => 'date',
					'order'   => 'ASC',
				);
				break;
			case 'date_desc':
			default:
				$order = array(
					'orderby' => 'date',
					'order'   => 'DESC',
				);
				break;
		}

		$args = array(
			'posts_per_page' => $count
		);
		$args = array_merge( $args, $order );

		if( ! empty( $category ) ) {
			$args['cat'] = $category_id;
		}

		$q = new WP_Query( $args );

		if ( in_array( $id, array( 'frontpage-widgets' ) ) ) {
			?><div class="row item-grid"><?php
		} else {
			$item_classes = '';
		}


		while ( $q->have_posts() ) {
			$q->the_post();
			$rating = get_post_meta( get_the_ID(), 'rating', true );
			?>
			<div class="<?php echo $item_classes; ?>">
				<div class="item">
					<?php if( has_post_thumbnail() ): ?>
						<figure class="item-thumb">
							<a href="<?php the_permalink(); ?>"><?php the_post_thumbnail(); ?></a>
							<?php if( ! empty( $rating ) ): ?>
								<span class="item-rating"><?php echo $rating; ?></span>
							<?php endif; ?>
						</figure>
					<?php endif; ?>

					<time class="item-time" datetime="<?php echo esc_attr( get_the_date( 'c' ) ); ?>"><?php echo get_the_date(); ?></time>

					<h4 class="item-title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h4>
					<?php the_excerpt(); ?>
				</div>
			</div>
			<?php
		}

		wp_reset_postdata();


		if ( in_array( $id, array( 'frontpage-widgets' ) ) ) {
			?></div><!-- /row item-grid --><?php
		}

		echo $after_widget;

	} // widget

	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		$instance['title']       = sanitize_text_field( $new_instance['title'] );
		$instance['category_id'] = intval( $new_instance['category_id'] );
		$instance['sort']        = sanitize_key( $new_instance['sort'] );
		$instance['count']       = absint( $new_instance['count'] );
		$instance['columns']     = intval( $new_instance['columns'] );

		return $instance;
	} // save

	function form($instance) {

		$instance = wp_parse_args( (array) $instance, array(
			'title'       => '',
			'category_id' => 0,
			'sort'        => 'date_desc',
			'count'       => 3,
			'columns'     => 3,
		) );

		$title       = $instance['title'];
		$category_id = $instance['category_id'];
		$sort        = $instance['sort'];
		$count       = $instance['count'];
		$columns     = $instance['columns'];

		echo '<p><label for="' . $this->get_field_id( 'title' ) . '">' . __( 'Title:', 'ci_theme' ) . '</label><input id="' . $this->get_field_id( 'title' ) . '" name="' . $this->get_field_name( 'title' ) . '" type="text" value="' . esc_attr( $title ) . '" class="widefat" /></p>';
		echo '<p><label for="' . $this->get_field_id( 'category_id' ) . '">' . __( 'Select a category to display posts from:', 'ci_theme' ) . '</label>';
		wp_dropdown_categories( array(
			'show_option_all'   => __('All categories', 'ci_theme'),
			'show_count'        => 1,
			'hide_empty'        => 1,
			'selected'          => $category_id,
			'hierarchical'      => 1,
			'name'              => $this->get_field_name( 'category_id' ),
			'id'                => $this->get_field_id( 'category_id' ),
		) );

		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'sort' ); ?>"><?php _e( 'Sort by:', 'ci_theme' ); ?></label>
			<select id="<?php echo $this->get_field_id( 'sort' ); ?>" name="<?php echo $this->get_field_name( 'sort' ); ?>">
				<option value="date_desc" <?php selected( 'date_desc', $sort ); ?>><?php _e( 'Date: Newest &rarr; Older', 'ci_theme' ); ?></option>
				<option value="date_asc" <?php selected( 'date_asc', $sort ); ?>><?php _e( 'Date: Oldest &rarr; Newer', 'ci_theme' ); ?></option>
				<option value="rand" <?php selected( 'rand', $sort ); ?>><?php _e( 'Random order', 'ci_theme' ); ?></option>
				<option value="rating_num_desc" <?php selected( 'rating_num_desc', $sort ); ?>><?php _e( 'Rating: 10 &rarr; 0', 'ci_theme' ); ?></option>
				<option value="rating_num_asc" <?php selected( 'rating_num_asc', $sort ); ?>><?php _e( 'Rating: 0 &rarr; 10', 'ci_theme' ); ?></option>
				<option value="rating_asc" <?php selected( 'rating_asc', $sort ); ?>><?php _e( 'Rating: A &rarr; Z', 'ci_theme' ); ?></option>
				<option value="rating_desc" <?php selected( 'rating_desc', $sort ); ?>><?php _e( 'Rating: Z &rarr; A', 'ci_theme' ); ?></option>
			</select>
		</p>

		<p><label for="<?php echo $this->get_field_id( 'count' ); ?>"><?php _e( 'Number of posts to show:', 'ci_theme' ); ?></label><input id="<?php echo $this->get_field_id( 'count' ); ?>" name="<?php echo $this->get_field_name( 'count' ); ?>" type="number" min="1" step="1" value="<?php echo esc_attr( $count ); ?>" class="widefat"/></p>

		<p>
			<label for="<?php echo $this->get_field_id( 'columns' ); ?>"><?php _e( 'Output Columns:', 'ci_theme' ); ?></label>
			<select id="<?php echo $this->get_field_id( 'columns' ); ?>" name="<?php echo $this->get_field_name( 'columns' ); ?>">
				<option value="2" <?php selected( 2, $columns ); ?>>2</option>
				<option value="3" <?php selected( 3, $columns ); ?>>3</option>
				<option value="4" <?php selected( 4, $columns ); ?>>4</option>
			</select>
		</p>
		<?php

	} // form

	private function _sort_subcategories_by_count( $a, $b ) {
		if( $a->count == $b->count ) {
			return 0;
		}
		return ( $a->count > $b->count ) ? 1 : -1;
	}

} // class


register_widget('CI_Category');

endif; // class_exists
?>